@@ -6,7 +6,7 @@ use File::Spec;
 use lib qw( lib );
 use ExtUtils::ParseXS::Constants ();
 
-our $VERSION = '3.18_03';
+our $VERSION = '3.18_04';
 
 our (@ISA, @EXPORT_OK);
 @ISA = qw(Exporter);
@@ -16,7 +16,6 @@ our (@ISA, @EXPORT_OK);
   C_string
   valid_proto_string
   process_typemaps
-  make_targetable
   map_type
   standard_XS_defs
   assign_func_args
@@ -43,7 +42,6 @@ ExtUtils::ParseXS::Utilities - Subroutines used with ExtUtils::ParseXS
     C_string
     valid_proto_string
     process_typemaps
-    make_targetable
     map_type
     standard_XS_defs
     assign_func_args
@@ -282,76 +280,6 @@ sub process_typemaps {
   return $typemap;
 }
 
-=head2 C<make_targetable()>
-
-=over 4
-
-=item * Purpose
-
-Populate C<%targetable>.  This constitutes a refinement of the output of
-C<process_typemaps()> with respect to its fourth output, C<$output_expr_ref>.
-
-=item * Arguments
-
-  %targetable = make_targetable($output_expr_ref);
-      
-Single hash reference:  the fourth such ref returned by C<process_typemaps()>.
-
-=item * Return Value
-
-Hash.
-
-=back
-
-=cut
-
-sub make_targetable {
-  my $output_expr_ref = shift;
-
-  our $bal; # ()-balanced
-  $bal = qr[
-    (?:
-      (?>[^()]+)
-      |
-      \( (??{ $bal }) \)
-    )*
-  ]x;
-
-  # matches variations on (SV*)
-  my $sv_cast = qr[
-    (?:
-      \( \s* SV \s* \* \s* \) \s*
-    )?
-  ]x;
-
-  my $size = qr[ # Third arg (to setpvn)
-    , \s* (??{ $bal })
-  ]x;
-
-  my %targetable;
-  foreach my $key (keys %{ $output_expr_ref }) {
-    # We can still bootstrap compile 're', because in code re.pm is
-    # available to miniperl, and does not attempt to load the XS code.
-    use re 'eval';
-
-    my ($type, $with_size, $arg, $sarg) =
-      ($output_expr_ref->{$key} =~
-        m[^
-          \s+
-          sv_set([iunp])v(n)?    # Type, is_setpvn
-          \s*
-          \( \s*
-            $sv_cast \$arg \s* , \s*
-            ( (??{ $bal }) )    # Set from
-          ( (??{ $size }) )?    # Possible sizeof set-from
-          \) \s* ; \s* $
-        ]x
-    );
-    $targetable{$key} = [$type, $with_size, $arg, $sarg] if $type;
-  }
-  return %targetable;
-}
-
 =head2 C<map_type()>
 
 =over 4
